@@ -86,6 +86,7 @@ from connexity_pipecat.core.voice_calls.templates import (
 from connexity_pipecat.core.voice_calls.twilio_service import TwilioClient
 from connexity_pipecat.helpers.get_model import get_llm_service
 from connexity_pipecat.helpers.append_message_history_to_function import append_message_history_to_function
+from connexity_pipecat.helpers.with_agent_id import with_agent_id
 from connexity_pipecat.helpers.perform_post_call_analytics import perform_post_call_analysis
 
 from connexity_pipecat.core.voice_calls.helpers.end_call import user_idle_end_call
@@ -248,6 +249,8 @@ async def websocket_endpoint(websocket: WebSocket, prefix: Optional[str] = None)
             volume=0.5,
         )
 
+        prefix = prefix or 'default'
+
         config = get_config(prefix)
         raw_vad_params = config.get("vad_params")
         vad_params = initiate_vad_params(raw_vad_params)
@@ -326,7 +329,7 @@ async def websocket_endpoint(websocket: WebSocket, prefix: Optional[str] = None)
         tts = get_tts_service(tts_settings)
 
         tools_meta = get_tool_manager().get_tools_schema(config["tools"])
-        function_handlers = get_tool_manager().get_handlers(config["tools"])
+        function_handlers = get_tool_manager().get_handlers(config["tools"].keys())
 
         context = OpenAILLMContext(message_history, tools=tools_meta)
         context_aggregator = llm_service.create_context_aggregator(context)
@@ -334,7 +337,7 @@ async def websocket_endpoint(websocket: WebSocket, prefix: Optional[str] = None)
         for name, raw_handler in function_handlers.items():
             llm_service.register_function(
                 name,
-                append_message_history_to_function(raw_handler, context)
+                with_agent_id(append_message_history_to_function(raw_handler, context), prefix)
             )
 
         pipeline_steps_list = [
@@ -493,6 +496,8 @@ async def platform_inference(request_body: RequestBody, prefix: Optional[str] =
     conversation_history.append(
         {"role": "user", "content": request_body.query})
 
+    prefix = prefix or 'default'
+
     config = get_config(prefix)
 
     llm_service = get_llm_service(
@@ -510,9 +515,13 @@ async def platform_inference(request_body: RequestBody, prefix: Optional[str] =
         {"role": "system", "content": system_prompt}, *conversation_history]
 
     tools_meta = get_tool_manager().get_tools_schema(config["tools"])
-    function_handlers = get_tool_manager().get_handlers(config["tools"])
-    for name, handler in function_handlers.items():
-        llm_service.register_function(name, handler)
+    function_handlers = get_tool_manager().get_handlers(config["tools"].keys())
+
+    for name, raw_handler in function_handlers.items():
+        llm_service.register_function(
+            name,
+            with_agent_id(append_message_history_to_function(raw_handler, context), prefix)
+        )
 
     context = OpenAILLMContext(message_history, tools=tools_meta)
     assistant_aggr = llm_service.create_context_aggregator(context).assistant()
@@ -563,9 +572,22 @@ async def platform_inference(request_body: RequestBody, prefix: Optional[str] =
         if any(v > 0 for v in stats.values())
     }
 
-    # normalise whitespace: collapse runs of any space‑like chars to a single “ ”
-    raw_reply = context.messages[-1]["content"]
-    clean_reply = re.sub(r"[ \u00A0\u202F]+", " ", raw_reply).strip()
+    # ───────────  newline & whitespace cleanup  ────────────
+    # 0) remove all newline characters that slipped through token streaming
+    raw_reply = context.messages[-1]["content"].replace("\n", "")
+
+    # Remove *any run* of ASCII / NBSP / thin‑NBSP that sits *inside* a word
+    no_intra_word_gaps = re.sub(
+        r'(?<=\w)[ \u00A0\u202F]+(?=\w)', '', raw_reply
+    )
+
+    # 2) strip spaces that appear immediately before punctuation
+    no_space_before_punct = re.sub(
+        r'[ \u00A0\u202F]+([,.;:!?])', r'\1', no_intra_word_gaps
+    )
+
+    # 3) collapse leftover runs of any space‑like char to a single ASCII space
+    clean_reply = re.sub(r'[ \u00A0\u202F]+', ' ', no_space_before_punct).strip()
     return JSONResponse(
         content={
             "model": get_config(prefix)["llm"]["main"]["model"],
